#include <vector>

#include "geos/geom/GeometryFactory.h"
#include "geos/geom/Geometry.h"
#include "geos/geom/Coordinate.h"
#include "geos/geom/CoordinateArraySequence.h"
#include "geos/geom/LinearRing.h"
#include "geos/geom/Point.h"
#include "geos/geom/Polygon.h"
#include "geos/geom/MultiPolygon.h"
#include <qDebug>
#include <QPainterPath>
#include <QWidget>
#include <QLabel>
#include <QPainter>
#include <QApplication>

QDebug &operator <<(QDebug &debug,const std::string &string){
    return debug<<QString::fromStdString(string);
}

using namespace geos::geom;
using namespace std;

QPainterPath toPainterPath(const geos::geom::Geometry *geometry){
    GeometryTypeId type=geometry->getGeometryTypeId();
    QPainterPath path;
    path.setFillRule(Qt::OddEvenFill);
    if(type==GEOS_MULTIPOLYGON){
        MultiPolygon *polygons=(MultiPolygon *)geometry;
        for(int i=0;i<polygons->getNumGeometries();i++){
            QPainterPath add=toPainterPath(polygons->getGeometryN(i));
            path.addPath(add);
        }
    }
    if(type==GEOS_POLYGON){
        Polygon *polygon=(Polygon *)geometry;
        qDebug()<<"area="<<polygon->getArea();
        const LineString *exterior=polygon->getExteriorRing();
        for(int i=0;i<exterior->getNumPoints();i++){
            Point *point=exterior->getPointN(i);
            const Coordinate *coordinate=point->getCoordinate();
            qDebug()<<"#"<<i<<" "<<coordinate->x<<","<<coordinate->y;
            if(i==0) path.moveTo(coordinate->x,coordinate->y);
            else path.lineTo(coordinate->x,coordinate->y);
        }
        path.closeSubpath();

        for(int j=0;j<polygon->getNumInteriorRing();j++){

            const LineString *interior=polygon->getInteriorRingN(j);
            for(int i=0;i<interior->getNumPoints();i++){
                Point *point=interior->getPointN(i);
                const Coordinate *coordinate=point->getCoordinate();
                qDebug()<<"#"<<i<<" "<<coordinate->x<<","<<coordinate->y;
                if(i==0) path.moveTo(coordinate->x,coordinate->y);
                else path.lineTo(coordinate->x,coordinate->y);
            }
            path.closeSubpath();
        }
    }
    return path;
}

void show(Geometry *geometry){
    QLabel *w=new QLabel;
    QPixmap pixmap(500,600);
    {
        QPainterPath path=toPainterPath(geometry);
        QPainter p(&pixmap);
        p.setBrush(QColor(0,255,0));
        p.scale(10,10);
        p.drawPath(path);
    }
    w->setPixmap(pixmap);

    w->show();

    while(w->isVisible()){
        qApp->processEvents();
    }
}


int main(int argc,char *argv[]) {
    QApplication app(argc,argv);

 GeometryFactory factory;

 vector< Geometry * > *polys1 = new vector<Geometry*>();
 vector< Geometry * > *polys2 = new vector<Geometry*>();
 vector< Geometry * > *holes1 = new vector<Geometry*>();

 CoordinateArraySequence coords1;
 coords1.add(Coordinate(1, 1));
 coords1.add(Coordinate(1, 5));
 coords1.add(Coordinate(5, 5));
 coords1.add(Coordinate(5, 1));
 coords1.add(Coordinate(1, 1));
 holes1->push_back( factory.createLinearRing() );
 polys1->push_back( factory.createPolygon(factory.createLinearRing(coords1), holes1) );

 CoordinateArraySequence coords2;
 coords2.add(Coordinate(3, 3));
 coords2.add(Coordinate(3, 4));
 coords2.add(Coordinate(4, 4));
 coords2.add(Coordinate(4, 3));
 coords2.add(Coordinate(3, 3));
 polys2->push_back( factory.createPolygon(factory.createLinearRing(coords2), new vector<Geometry*>) );

 MultiPolygon *mpoly1 = factory.createMultiPolygon(polys1);
 MultiPolygon *mpoly2 = factory.createMultiPolygon(polys2);

 qDebug() << "      Mpoly1: " << mpoly1->toString();
 qDebug() << "      Mpoly2: " << mpoly2->toString();
 Geometry *intersection = mpoly1->intersection(mpoly2);
 qDebug() << "Intersection: " << intersection->toString();
 Geometry *united = mpoly1->Union(mpoly2);
 qDebug() << "United: " << united->toString() ;
 Geometry *subtr= mpoly1->difference(mpoly2);
 qDebug() << "Substr: " << subtr->toString() ;

// show(intersection);
 show(subtr);


 delete mpoly1;
 delete mpoly2;
 delete intersection;
}
